using FitConnect.Services.Models;
using FitConnect.Services.Models.v1.Submission;

namespace FitConnect.Models;

public class Submission {
    public string Id { get; set; } = null!;
    public string CaseId { get; set; } = null!;
    public Destination Destination { get; set; } = new();

    public string DestinationId {
        get => Destination.DestinationId ?? throw new ArgumentNullException(nameof(DestinationId));
        set => Destination.DestinationId = value;
    }

    public List<Attachment> Attachments { get; set; } = new();
    public List<string> AttachmentIds { get; set; } = new();

    public ServiceType ServiceType { get; set; } = new();

    public Callback? Callback { get; set; }
    public Api.Metadata.Metadata? Metadata { get; set; }
    public string? Data { get; set; }
    public string? EncryptedMetadata { get; set; }
    public string? EncryptedData { get; set; }
    public string? MetaAuthentication => EncryptedMetadata?.Split('.').Last();
    public string? DataAuthentication => EncryptedData?.Split('.').Last();

    public bool IsSubmissionReadyToAdd(out string? error) {
        var innerError = "";
        if (string.IsNullOrEmpty(DestinationId)) innerError += "DestinationId is required\r\n";

        if (ServiceType.IsValid()) innerError += "ServiceType is invalid\r\n";

        if (string.IsNullOrWhiteSpace(innerError)) {
            error = null;
            return true;
        }

        error = innerError.Trim();
        return false;
    }

    public bool IsSubmissionReadyToSend() {
        return true;
    }

    public static implicit operator SubmissionForPickupDto(Submission sub) {
        return new SubmissionForPickupDto {
            Id = sub.Id,
            CaseId = sub.CaseId,
            DestinationId = sub.DestinationId
        };
    }

    public static explicit operator Submission(SubmissionForPickupDto dto) {
        return new Submission {
            Id = dto.Id,
            Callback = null,
            DestinationId = dto.DestinationId ??
                            throw new NullReferenceException(nameof(dto.DestinationId)),
            ServiceType = new ServiceType()
        };
    }

    public static explicit operator Submission(SubmissionDto dto) {
        return new Submission {
            Id = dto.SubmissionId,
            Callback = dto.Callback == null ? null : (Callback)dto.Callback,
            DestinationId = dto.DestinationId ??
                            throw new NullReferenceException(nameof(dto.DestinationId)),
            ServiceType =
                dto.ServiceType == null ? new ServiceType() : (ServiceType)dto.ServiceType,
            EncryptedData = dto.EncryptedData,
            EncryptedMetadata = dto.EncryptedMetadata,
            AttachmentIds = dto.Attachments ?? new List<string>(),
            CaseId = dto.CaseId
        };
    }

    public static explicit operator SubmitSubmissionDto(Submission dto) {
        return new SubmitSubmissionDto {
            EncryptedData = dto.EncryptedData,
            EncryptedMetadata = dto.EncryptedMetadata
        };
    }

    public static explicit operator CreateSubmissionDto(Submission sub) {
        var result = new CreateSubmissionDto {
            AnnouncedAttachments = sub.Attachments.Select(a => a.Id).ToList(),
            DestinationId = sub.DestinationId
        };
        if (sub.ServiceType != null)
            result.ServiceType = (ServiceTypeDto)sub.ServiceType;
        if (sub.Callback != null) result.Callback = (CallbackDto)sub.Callback;

        return result;
    }

    internal Submission() {
    }
}

